<!DOCTYPE html>
<html>
  <head>
    <title>Dash Hudson API Documentation</title>
    <meta charset="utf-8"/>
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <link href="https://fonts.googleapis.com/css?family=Muli:300,400,700" rel="stylesheet">
    <link rel="stylesheet" href="https://developer.dashhudson.com/assets/styles.css">
    <link rel=icon href="https://developer.dashhudson.com/assets/favicon.png">
  </head>
  <body>
    <redoc spec-url="openapi.json"></redoc>
    <script src="https://cdn.jsdelivr.net/npm/redoc@next/bundles/redoc.standalone.js"> </script>
  </body>
</html>
